#!/usr/bin/env python

##############################################
"""
JB-RPINM
Jarno Baselier - Raspberry Pi Network Monitor
https://jarnobaselier.nl/

Version: 1.0
Date: 01-02-2020

License: Free use and distribution but never 
change this header of in-program "about" section!

This application is a portable Network Monitor
especially created and designed for the Raspberry
Pi 4 with 7.0 inch display. The JB-RPINM is a 
Python-based tool to see and test the network
statistics. Feel free to use and re-distribute
this software but never change copyright items.

If you want to help developing an awesome
JB-RPINM V2 with better functions, coding and
functionality please feel free to contact me.
I can use all the help I can get :-) 
"""
#############################################

###############
#Top Variables#
###############

#Working Directory
workingdir = "/root/Desktop/JBRPINM/"

#Working Directory
workinglootdir = workingdir + "loot/"

####################
#import Section#
####################

#Import GI / GTK
import gi
gi.require_version("Gtk", "3.0")
from gi.repository import Gtk

#Import System Libraries
import sys
import os
import socket
import subprocess
import psutil
from datetime import datetime
import math
import re #RegEx

#Import Logging
import logging

#Import IP Manipulation Libraries
import ipaddr #IPADDR
from netaddr import IPAddress #IPAddress function from netaddr library
import netifaces #NetIFaces
#import scapy #for later use - now disabled
#import pyshark #for later use - now disabled
import nmap

#Import URL Libraries:
from urllib.request import Request, urlopen
from urllib.error import  URLError

#Import Speedtest
import speedtest

#Import Plumbum
from plumbum.cmd import grep #For using the GREP command

#########################
#Define Global Functions#
#########################

#Set date and time
now = datetime.now()
timestamp = datetime.timestamp(now)
nowdate = (now.strftime("%Y-%m-%d"))
nowdatetime = (now.strftime("%Y-%m-%d-%H-%M-%S"))

#Function to check for external IP:
def ip_url(url, attempts):
    for attempt in range(attempts):
        try:
            req = Request(url)
            response = urlopen(req)
        except URLError as e:
            if hasattr(e, 'reason'):
                return "No ext. IP found"
            elif hasattr(e, 'code'):
                return "No ext. IP found"
        else:
            return response.read().decode('utf8')

    return None
	
#Ping function
def ping_function(host, attempts):
	response = os.system("ping " + host + " -c " + attempts + " -b")
	if response == 0:
		return "Yes"
	else:
		return "No"

#Define Default NIC
def defnic_function():
	global defnic
	defnic = str(netifaces.gateways()['default'][netifaces.AF_INET][1])

#Define Default NIC IP
def defnic_ipaddr_function():
	global defnic_ipaddr
	defnic_ipaddr = netifaces.ifaddresses(defnic)[netifaces.AF_INET][0]['addr']
	if defnic_ipaddr is None:
		defnic_ipaddr = str("No IP")

#Define Default NIC IP Type IPv4 (4) or IPv6 (6)
def defnic_ipaddr_type_function():
	global ipaddr_type
	ipaddr_type = ipaddr.IPAddress(defnic_ipaddr)
	ipaddr_type = ('IP%s' % (ipaddr_type.version))
	if ipaddr_type is None:
		ipaddr_type = str("No IP")

#Define Default NIC Subnetmask
def defnic_subnet_function():
	global defnic_subnet
	defnic_subnet = netifaces.ifaddresses(defnic)[netifaces.AF_INET][0]['netmask']
	if defnic_subnet is None:
		defnic_subnet = str("No Subnet")

#Define Default NIC Subnetmask CIDR
def defnic_subnet_cidr_function():
	global defnic_subnet_cidr
	defnic_subnet_cidr = IPAddress(defnic_subnet).netmask_bits()
	if defnic_subnet_cidr is None:
		defnic_subnet_cidr = str("No Subnet")

#Define Default IP CIDR Notation
def def_cidr_ip_function():

	global cidr_ip
	cidr_ip = defnic_ipaddr + "/" + str(defnic_subnet_cidr)
	if cidr_ip is None:
		cidr_ip = str("No CIDR-IP")
		
#Define Default NIC Gateway
def defnic_gateway_function():
	global defnic_gateway
	defnic_gateway = netifaces.ifaddresses(defnic)[netifaces.AF_INET][0]['broadcast']
	if defnic_gateway is None:
		defnic_gateway = str("No Gateway")

#Define External IPv4 addresses
def external_ipv4_function():
	global external_ipv4
	external_ipv4 = str((ip_url('https://api.ipify.org', 1)))
	if external_ipv4 is None:
		external_ipv4 = str("No ext. IPv4 found")

#Define External IPv6 addresses
def external_ipv6_function():
	global external_ipv6
	external_ipv6 = str((ip_url('https://api6.ipify.org', 1)))
	if external_ipv6 is None:
		external_ipv6 = str("No ext. IPv6 found")

#Define Get ETHTool Output and post it to tempfile to work with
def ethtool_output():	
	ethtool_output = os.system("ethtool eth0 >>" + workingdir + "ethoutput.txt")
		
#Define if ETH is connected
def eth_connected():
	global eth_connected_status
	eth_connected_status = os.system("ethtool eth0 | grep \"Link detected: yes\"")
	if eth_connected_status == 0:
		eth_connected_status = "Yes"
	else:
		eth_connected_status = "No"

#Get ETH Duplex Mode
def eth_duplex():
	#Is ETH0 Connected
	eth_connected()
	if (eth_connected_status == 'Yes'):
	
		#Remove if ETHTool Tempfile exists
		if os.path.exists(workingdir + "ethoutput.txt"):
			os.remove(workingdir + "ethoutput.txt")
			
		#Create new ETHTool Tempfile
		ethtool_output()
		
		global eth_duplex_status
		
		eth_duplex_status = (grep("Duplex", workingdir + "ethoutput.txt"))
		eth_duplex_status = eth_duplex_status.replace('\tDuplex: ', '')
		eth_duplex_status = eth_duplex_status.replace('\n', '')
		
		#Remove if ETHTool Tempfile exists
		if os.path.exists(workingdir + "ethoutput.txt"):
			os.remove(workingdir + "ethoutput.txt")
	else:
		eth_duplex_status = "-"

#Get ETH Speed
def eth_speed():
	#Is ETH0 Connected
	eth_connected()
	if (eth_connected_status == 'Yes'):
	
		#Remove if ETHTool Tempfile exists
		if os.path.exists(workingdir + "ethoutput.txt"):
			os.remove(workingdir + "ethoutput.txt")
			
		#Create new ETHTool Tempfile
		ethtool_output()
		
		global eth_speed_status
		
		eth_speed_status = (grep("Speed", workingdir + "ethoutput.txt"))
		eth_speed_status = eth_speed_status.replace('\tSpeed: ', '')
		eth_speed_status = eth_speed_status.replace('\n', '')
		
		#Remove if ETHTool Tempfile exists
		if os.path.exists(workingdir + "ethoutput.txt"):
			os.remove(workingdir + "ethoutput.txt")
			
	else:
		eth_speed_status = "-"

#Define Get ETHTool Output and post it to tempfile to work with
def iwlist_output():	
	iwlist_output = os.system("iwlist wlan0 channel >>" + workingdir + "iwlist.txt")

def iwconfig_output():	
	iwconfig_output = os.system("iwconfig >>" + workingdir + "iwconfigoutput.txt")
	
def wifi_connected():
	global wifi_connected_status
	wifi_connected_status = os.system("iwgetid | grep \"wlan0\"")
	if wifi_connected_status == 0:
		wifi_connected_status = "Yes"
	else:
		wifi_connected_status = "No"
		
		
def wifi_channel():
	#Is WiFi Connected
	wifi_connected()
	if (wifi_connected_status == 'Yes'):
	
		#Remove if IWList Tempfile exists
		if os.path.exists(workingdir + "iwlist.txt"):
			os.remove(workingdir + "iwlist.txt")
			
		#Create new IWList Tempfile
		iwlist_output()
		
		global wifi_channel_status
		
		wifi_channel_status = (grep("(Channel ", workingdir + "iwlist.txt"))
		wifi_channel_status = wifi_channel_status.strip() #Remove all leading and trailing spaces
		wifi_channel_status = wifi_channel_status.split('Channel ', 1)[-1] #Split the line and take the second part
		wifi_channel_status = wifi_channel_status.replace(')', '') #Remove last closing bracket
		
		#Remove if ETHTool Tempfile exists
		if os.path.exists(workingdir + "iwlist.txt"):
			os.remove(workingdir + "iwlist.txt")
			
	else:
		wifi_channel_status = "-"

def wifi_frequency():
	#Is WiFi Connected
	wifi_connected()
	if (wifi_connected_status == 'Yes'):
	
		#Remove if IWConfig Tempfile exists
		if os.path.exists(workingdir + "iwconfigoutput.txt"):
			os.remove(workingdir + "iwconfigoutput.txt")
			
		#Create new IWConfig Tempfile
		iwconfig_output()
		
		global wifi_frequency_status
		
		wifi_frequency_status = (grep("Frequency", workingdir + "iwconfigoutput.txt"))
		wifi_frequency_status = wifi_frequency_status.strip() #Remove all leading and trailing spaces
		wifi_frequency_status = wifi_frequency_status.split('Frequency:', 1)[-1] #Split the line and take the second part
		wifi_frequency_status = wifi_frequency_status.split(' Access', 1)[0] #Split the line and take the second part
		
		#Remove if IWConfig Tempfile exists
		if os.path.exists(workingdir + "iwconfigoutput.txt"):
			os.remove(workingdir + "iwconfigoutput.txt")
			
	else:
		wifi_frequency_status = "-"

def wifi_accesspointmac():
	#Is WiFi Connected
	wifi_connected()
	if (wifi_connected_status == 'Yes'):
	
		#Remove if IWConfig Tempfile exists
		if os.path.exists(workingdir + "iwconfigoutput.txt"):
			os.remove(workingdir + "iwconfigoutput.txt")
			
		#Create new IWConfig Tempfile
		iwconfig_output()
		
		global wifi_accesspointmac_status
		
		wifi_accesspointmac_status = (grep("Access Point", workingdir + "iwconfigoutput.txt"))
		wifi_accesspointmac_status = wifi_accesspointmac_status.strip() #Remove all leading and trailing spaces
		wifi_accesspointmac_status = wifi_accesspointmac_status.split('Access Point: ', 1)[-1] #Split the line and take the second part
		
		#Remove if IWConfig Tempfile exists
		if os.path.exists(workingdir + "iwconfigoutput.txt"):
			os.remove(workingdir + "iwconfigoutput.txt")
			
	else:
		wifi_accesspointmac_status = "-"

def wifi_bitrate():
	#Is WiFi Connected
	wifi_connected()
	if (wifi_connected_status == 'Yes'):
	
		#Remove if IWConfig Tempfile exists
		if os.path.exists(workingdir + "iwconfigoutput.txt"):
			os.remove(workingdir + "iwconfigoutput.txt")
			
		#Create new IWConfig Tempfile
		iwconfig_output()
		
		global wifi_bitrate_status
		
		wifi_bitrate_status = (grep("Bit Rate", workingdir + "iwconfigoutput.txt"))
		wifi_bitrate_status = wifi_bitrate_status.strip() #Remove all leading and trailing spaces
		wifi_bitrate_status = wifi_bitrate_status.split('Bit Rate=', 1)[-1] #Split the line and take the second part
		wifi_bitrate_status = wifi_bitrate_status.split(' Tx', 1)[0] #Split the line and take the second part
		
		#Remove if IWConfig Tempfile exists
		if os.path.exists(workingdir + "iwconfigoutput.txt"):
			os.remove(workingdir + "iwconfigoutput.txt")
			
	else:
		wifi_bitrate_status = "-"

def wifi_link_quality():
	#Is WiFi Connected
	wifi_connected()
	if (wifi_connected_status == 'Yes'):
	
		#Remove if IWConfig Tempfile exists
		if os.path.exists(workingdir + "iwconfigoutput.txt"):
			os.remove(workingdir + "iwconfigoutput.txt")
			
		#Create new IWConfig Tempfile
		iwconfig_output()
		
		global wifi_link_quality_status
		
		wifi_link_quality_status = (grep("Link Quality", workingdir + "iwconfigoutput.txt"))
		wifi_link_quality_status = wifi_link_quality_status.strip() #Remove all leading and trailing spaces
		wifi_link_quality_status = wifi_link_quality_status.split('Link Quality=', 1)[-1] #Split the line and take the second part
		wifi_link_quality_status = wifi_link_quality_status.split(' Signal', 1)[0] #Split the line and take the second part
		
		#Remove if IWConfig Tempfile exists
		if os.path.exists(workingdir + "iwconfigoutput.txt"):
			os.remove(workingdir + "iwconfigoutput.txt")
			
	else:
		wifi_link_quality_status = "-"

def nmap_scan(scantype):
		portsMostUsed = "21,22,23,25,53,80,110,111,135,139,143,443,445,993,995,1723,3306,3389,5900,8080"
		portsDefault = "1-1001"
		portsAll = "1-65535"
		
		if scantype == "normalmode":
			nm.scan(hosts=cidr_ip, arguments='-v -p' + portsMostUsed)
		elif scantype == "hostmode":
			nm.scan(hosts=cidr_ip, arguments='-v -sL')
		elif scantype == "silentmode":
			nm.scan(hosts=cidr_ip, arguments='-v -sS -p' + portsDefault)
		elif scantype == "aggessivemode":
			nm.scan(hosts=cidr_ip, arguments='-v -A -sC -O -sV -sU --version-intensity 5 -f -p' + portsDefault)

		#Print info to console
		print ("NMAP Scan Performed with command: " + nm.command_line())
		for host in nm.all_hosts():
			print('----------------------------------------------------')
			print('Host : %s (%s)' % (host, nm[host].hostname()))
			print('State : %s' % nm[host].state())
			for proto in nm[host].all_protocols():
				print('----------')
				print('Protocol : %s' % proto)
				lport = sorted(nm[host][proto].keys())
				for port in lport:
					print ('port : %s\tstate : %s' % (port, nm[host][proto][port]['state']))
			print('----------------------------------------------------')
		#Add information to logfile
		logging.info ("NMAP Scan Performed with command: " + nm.command_line())
		for host in nm.all_hosts():
			logging.info('----------------------------------------------------')	
			logging.info('Host : %s (%s)' % (host, nm[host].hostname()))
			logging.info('State : %s' % nm[host].state())
			for proto in nm[host].all_protocols():
				logging.info('----------')
				logging.info('Protocol : %s' % proto)
				lport = sorted(nm[host][proto].keys())
				for port in lport:
					logging.info('port : %s\tstate : %s' % (port, nm[host][proto][port]['state']))
			logging.info('----------------------------------------------------')
		#Add information to seperate logfile in loot dir
		nmaplogfile = open(workinglootdir + "nmap-" + str(nowdatetime) + "-" + str(defnic_ipaddr) + ".txt","w+")
		nmaplogfile.write ("NMAP Scan Performed with command: " + nm.command_line() + "\n")
		for host in nm.all_hosts():
			nmaplogfile.write('----------------------------------------------------\n')
			nmaplogfile.write('Host : %s (%s)\n' % (host, nm[host].hostname()))
			nmaplogfile.write('State : %s\n' % nm[host].state())
			for proto in nm[host].all_protocols():
				nmaplogfile.write('----------\n')
				nmaplogfile.write('Protocol : %s\n' % proto)
				lport = sorted(nm[host][proto].keys())
				for port in lport:
					nmaplogfile.write ('port : %s\tstate : %s\n' % (port, nm[host][proto][port]['state']))
			nmaplogfile.write('----------------------------------------------------\n')
		nmaplogfile.close()
		#Export information to CSV
		print(nm.csv(),file=open(workinglootdir + "nmap-" + str(nowdatetime) + "-" + str(defnic_ipaddr) + ".csv",'a'))

def pcapcreation(pcaptime):
		global pcapname
		pcapname = "pcap-" + str(nowdatetime) + "-" + str(defnic_ipaddr) + ".pcap"

		if pcaptime == 3:
			pcaprecordtime = "10"
		elif pcaptime == 10:
			pcaprecordtime = "10"

		subprocess.Popen(["timeout", pcaprecordtime, "tcpdump", "-s", "0", "-i", defnic, "-w", workinglootdir + pcapname], stdout=subprocess.PIPE)

def netdiscover_scan():
		global netdiscovername
		netdiscovername = "netdiscover-arp-" + str(nowdatetime) + "-" + str(defnic_ipaddr) + ".txt"
		netdiscoverlogpath = open(workinglootdir + netdiscovername, "a")
		subprocess.Popen(["timeout", "120", "netdiscover", "-i", defnic, "-P"], stdout=netdiscoverlogpath)
		
	
#########################
#Settings#
#You may change these settings if required#
#########################

#Enable Logging
enable_logging = 1

if enable_logging == 1:
	logging.basicConfig(level=logging.INFO, filename=workingdir + "logfile.txt", filemode="a+",
	format="%(asctime)-15s %(levelname)-8s %(message)s")
else:
	os.remove("logfile.txt")

#Enable Default NIC at startup
defnic_function()
eth_connected()
wifi_connected()

enable_eth0_as_def_nic = 1
if (enable_eth0_as_def_nic == 1 and defnic == 'wlan0' and eth_connected_status == 'Yes'):
	subprocess.call(["ifmetric", "wlan0", "600"])
	subprocess.call(["ifmetric", "eth0", "100"])
	#Print info to console
	print("Changed default NIC before starting application to ETH0")
	#Add information to logfile
	logging.info("Changed default NIC before starting application to ETH0")

enable_wlan0_as_def_nic = 0
if (enable_wlan0_as_def_nic == 1 and defnic == 'eth0' and wifi_connected_status == 'Yes'):
	subprocess.call(["ifmetric", "eth0", "600"])
	subprocess.call(["ifmetric", "wlan0", "100"])
	#Print info to console
	print("Changed default NIC before starting application to WLAN0")
	#Add information to logfile
	logging.info("Changed default NIC before starting application to WLAN0")
	
	
	
#########################
#Define Global Variables#
#########################



#Location of the Glade XML File:
gladefile = workingdir + "JBRPINM.glade"

#Define Default NIC variable
defnic_function()

#Define Default NIC IP variable
defnic_ipaddr_function()

#Define Default NIC IP type variable
defnic_ipaddr_type_function()

#Define Default NIC Subnetmask variable
defnic_subnet_function()

#Define Default NIC Subnetmask CIDR variable
defnic_subnet_cidr_function()

#Define Default IP + CIDR notation - like 192.168.1.1\24
def_cidr_ip_function()

#Define Default NIC Gateway variable
defnic_gateway_function()
	
#Define External IPv4 addresses variable
external_ipv4_function()

#Define External IPv6 addresses variable
external_ipv6_function()

#Define if gateway is reachable / LAN is up
gateway_reachable = str((ping_function(defnic_gateway, "1")))

#Define if internet is reachable / WAN is up
internet_reachable = str((ping_function("8.8.8.8", "1")))

#Is ETH0 Connected
eth_connected()
		
#Define Duplex Mode
eth_duplex()

#Define Speed Mode
eth_speed()

#Is WLAN0 Connected
wifi_connected()
		
#Define WiFI Channel
wifi_channel()

#Define WiFi Frequency
wifi_frequency()

#Define WiFi AccessPoint Mac
wifi_accesspointmac()

#Define WiFi BitRate
wifi_bitrate()

#Define WiFi Link Quality
wifi_link_quality()

#Define NMAP Command
nm = nmap.PortScanner()


#########################
#Remove All Temp Files#
#########################

#EthTool Output
if os.path.exists(workingdir + "ethoutput.txt"):
    os.remove(workingdir + "ethoutput.txt")

#IWList Output
if os.path.exists(workingdir + "iwlist.txt"):
    os.remove(workingdir + "iwlist.txt")

#Remove if IWConfig Tempfile exists
if os.path.exists(workingdir + "iwconfigoutput.txt"):
	os.remove(workingdir + "iwconfigoutput.txt")


####################
#Start Jarno class#
#Initialize application#
####################
		
class Jarno:

	# Let's initialize the signals
	def __init__(self):
		#Glade / GTK Function
		self.gladefile = gladefile # store the file name
		self.builder = Gtk.Builder() # create an instance of the gtk.Builder
		self.builder.add_from_file(self.gladefile) # add the xml file to the Builder

		#Set internal IP information Label
		self.labelipinternal = self.builder.get_object("label_ipaddress_internal_dynamic") #Set label field
		self.labelipinternal.set_text(defnic + "\n" + defnic_ipaddr + "\n" + defnic_subnet + " - (" + str(defnic_subnet_cidr) + ")" + "\n" + defnic_gateway) #Set label content
		
		#Set IPv4 en IPv6 External IP Label
		self.labelipv4external = self.builder.get_object("label_ipv4address_external_dynamic") #Set label field
		self.labelipv4external.set_text(external_ipv4 + "\n" + external_ipv6) #Set label content
		
		#Connection Up / Ping info
		self.labelconnectionup = self.builder.get_object("label_connectionup_info") #Set label field
		self.labelconnectionup.set_text(gateway_reachable + "\n" + internet_reachable) #Set label content
		
		#Setting Speedtest Info
		self.labelspeedtestinfo = self.builder.get_object("label_speedtest_info") #Set label field
		self.labelspeedtestinfo.set_text("Run Speedtest to see results" + "\n" + "Run Speedtest to see results") #Set label content
		
		#Setting Ethernet Info
		self.labelethernetinfo = self.builder.get_object("label_ethernet_status_dynamic") #Set label field
		self.labelethernetinfo.set_text(eth_connected_status + "\n" + eth_duplex_status + "\n" + eth_speed_status) #Set label content
		
		#Setting Wireless Info
		self.labelwifiinfo = self.builder.get_object("label_wireless_status_dynamic") #Set label field
		self.labelwifiinfo.set_text(wifi_connected_status + "\n" + wifi_channel_status + "\n" + wifi_frequency_status + "\n" + wifi_accesspointmac_status + "\n" + wifi_bitrate_status + "\n" + wifi_link_quality_status) #Set label content
		
		#Define About Dialog
		self.aboutdialog = self.builder.get_object("JBRPINM-About")

		# This line does the magic of connecting the signals created in the Glade3
		# builder to our defines above. You must have one def for each signal if
		# you use this line to connect the signals.
		self.builder.connect_signals(self)

		# Start program / mainwindow
		self.window = self.builder.get_object("mainwindow") # This gets the 'mainwindow' object
		self.window.show() # this shows the 'mainwindow' object
		self.window.fullscreen() # this shows the 'mainwindow' in fullscreen

		#Add information to logfile
		logging.info("====================Starting a new session with the following info====================\n" + "\nDefault NIC: " + defnic + "\nInternal IPv4 Address: " + defnic_ipaddr + "\nSubnet Mask: " + defnic_subnet + "\nExternal IPv4 Address: " + external_ipv4 + "\nExternal IPv6 Address: " + external_ipv6)

#####################################
#Start Jarno Definitions / Functions#
#####################################

### MAINMENU / TOPMENU ###
	#Define on_mainwindow_destroy signal
	def on_mainwindow_destroy(self, object, data=None):
		Gtk.main_quit()
		#Print info to console
		print ("Quit with cancel")
		#Add information to logfile
		logging.info("====================Quit with cancel / cross====================")

	#Define on_gtk_quit_activate signal
	def on_gtk_quit_activate(self, menuitem, data=None):
		Gtk.main_quit()
		#Print info to console
		print ("Quit from menu")
		#Add information to logfile
		logging.info("====================Quit from menu====================")

	#Define the About dialog
	def on_gtk_about_activate(self, menuitem, data=None):
		self.response = self.aboutdialog.run()
		self.aboutdialog.hide()
		#Print info to console
		print ("Accessed About dialog")
		#Add information to logfile
		logging.info("Accessed About dialog")

### LEFT SIDE BUTTONS ###

	#Define Refresh button
	def on_button_refresh_clicked(self, button, data=None):
		###Refresh Internal IP###
		self.refresh_internal_ip_information()
		
		###Refresh External IP###
		self.refresh_external_ip_information()
				
		###Refresh Ping Statistics###
		self.refresh_ping_information()
		
		###Refresh Ethernet Connected Statistics###		
		self.refresh_ethernet_status_information()

		###Refresh WiFi Connected Statistics###		
		self.refresh_wifi_status_information()
		
	#Change NIC Button			
	def on_button_changenicfile_clicked(self, button, data=None):
		global defnic #import global variable into class for use
		if defnic == "eth0":
			subprocess.call(["ifmetric", "eth0", "600"])
			subprocess.call(["ifmetric", "wlan0", "100"])
		if defnic == "wlan0":
			subprocess.call(["ifmetric", "wlan0", "600"])
			subprocess.call(["ifmetric", "eth0", "100"])

		defnic_function()
		if defnic is None:
			defnic = str("No active network card found")
		
		#Print info to console
		print ("Change NIC Button - Changed primary NIC to: " + defnic)
		#Add information to logfile
		logging.info("Change NIC Button - Changed primary NIC to: " + defnic)
		
		#Refresh all statistics
		self.refresh_internal_ip_information()
		
	#Speedtest Button
	def on_button_speedtest_clicked(self, button, data=None):
		responseping = os.system("ping 8.8.8.8 -c 1 -b") #Perform a ping to check internet connection
		if responseping == 0:
			speedtester = speedtest.Speedtest()
			speedtester.get_best_server()

			downloadspeedbytes = speedtester.download()
			uploadspeedbytes = speedtester.upload()
			downloadspeedmbytes = downloadspeedbytes //1048776
			uploadspeedmbytes = uploadspeedbytes //1048776
				
			#Set new information
			self.labelspeedtestinfo.set_text(str(downloadspeedmbytes) + " Mbps" + "\n" + str(uploadspeedmbytes) + " Mbps")
				
			#Print info to console
			print ("Performed Speedtest with these results:" + "\n" + "Download: " + str(downloadspeedmbytes) + " Mbps" + "\n" + "Upload: " + str(uploadspeedmbytes) + " Mbps")
			#Add information to logfile
			logging.info("Performed Speedtest with these results: " + "Download: " + str(downloadspeedmbytes) + " Mbps - Upload: " + str(uploadspeedmbytes) + " Mbps")
		else:
			self.labelspeedtestinfo.set_text("No Internet Connection!")
		
	#Test Button
	def on_gtk_test_activate(self, button, data=None):

		#Print info to console
		print ("Accessed Page 2 - The Tools")
		#Add information to logfile
		logging.info("Accessed Page 2 - The Tools")

	#Define Empty Logfile button
	def on_button_emptylogfile_clicked(self, button, data=None):
		#Open and empty file
		open(workingdir + 'logfile.txt', 'w').close()
		#Print info to console
		print ("Cleaned logfile")
		#Add information to logfile
		logging.info("====================Cleared Logfile====================")

	#Define on_gtk_quit button_activate signal
	def on_gtk_quitbutton_activate(self, button, data=None):
		Gtk.main_quit()
		#Print info to console
		print ("Quit from button")
		#Add information to logfile
		logging.info("====================Quit from button====================")

	#Reboot Button	
	def on_gtk_rebootbutton_activate(self, button, data=None):
		subprocess.call(["reboot", "now"])
		#Add information to logfile
		logging.info("====================Reboot from button====================")
		
	#Shutdown Button	
	def on_gtk_shutdownbutton_activate(self, button, data=None):
		subprocess.call(["shutdown", "now"])
		#Add information to logfile
		logging.info("====================Shutdown from button====================")

### TOOLS BUTTONS ###

	#Define PCAP Button
	def on_button_start_pcap_activate(self, button, data=None):
		pcapcreation(3)
		#Print info to console
		print ("Created a PCAP file in the loot directory called: " + pcapname)
		#Add information to logfile
		logging.info("Created a PCAP file in the loot directory called: " + pcapname)	
		
	def on_button_start_pcap_10_activate(self, button, data=None):
		pcapcreation(10)
		#Print info to console
		print ("Created a PCAP file in the loot directory called: " + pcapname)
		#Add information to logfile
		logging.info("Created a PCAP file in the loot directory called: " + pcapname)

	#Define Normal NMAP Button
	def on_button_nmap_normal_clicked(self, button, data=None):
		nmap_scan("normalmode")

	#Define Host Discovery NMAP Button
	def on_button_nmap_host_clicked(self, button, data=None):
		nmap_scan("hostmode")
		
	#Define Silent NMAP Button
	def on_button_nmap_silent_clicked(self, button, data=None):
		nmap_scan("silentmode")
		
	#Define Aggressive NMAP Button
	def on_button_nmap_aggressive_clicked(self, button, data=None):
		nmap_scan("aggessivemode")

	#Define Netscanner Button
	def on_button_netscanner_clicked(self, button, data=None):
		
		netdiscover_scan()
	
		#Print info to console
		print ("Created a NetDiscover file in the loot directory called: " + str(netdiscovername))
		#Add information to logfile
		logging.info("Created a NetDiscover file in the loot directory called: " + str(netdiscovername))

		
### REFRESH FUNCTIONS ###	

	def refresh_internal_ip_information(self):
		#Define Default NIC variable
		defnic_function()

		#Define Default NIC IP variable
		defnic_ipaddr_function()

		#Define Default NIC IP type variable
		defnic_ipaddr_type_function()		

		#Define Default NIC Subnetmask variable
		defnic_subnet_function()

		#Define Default NIC Subnetmask CIDR variable
		defnic_subnet_cidr_function()
		
		#Define Default IP + CIDR notation - like 192.168.1.1\24
		def_cidr_ip_function()
		
		#Define Default NIC Gateway variable
		defnic_gateway_function()
		
		#Is ETH0 Connected
		eth_connected()
		
		#Define Duplex Mode
		eth_duplex()

		#Define Speed Mode
		eth_speed()

		#Set new information
		self.labelipinternal.set_text(defnic + "\n" + defnic_ipaddr + "\n" + defnic_subnet + " - (" + str(defnic_subnet_cidr) + ")"  + "\n" + defnic_gateway)
		self.labelethernetinfo.set_text(eth_connected_status + "\n" + eth_duplex_status + "\n" + eth_speed_status)
		#Print info to console
		print ("Refreshed Internal IP information to:" + "\n" + "Default NIC: " + defnic + "\n" + "IP address: " + defnic_ipaddr + "\n" + "Subnet Mask: " + defnic_subnet + " - (" + str(defnic_subnet_cidr) + ")"  + "\n" + "Gateway: " + defnic_gateway + "\n" + "Eth0 Connected: " + eth_connected_status + "\n" + "Eth0 Duplex: " + eth_duplex_status + "\n" + "Eth0 Speed: " + eth_speed_status)
		#Add information to logfile
		logging.info("Refreshed Internal IP information to:" + " - Default NIC: " + defnic + " - IP address: " + defnic_ipaddr + " - Subnet Mask: " + defnic_subnet + " - (" + str(defnic_subnet_cidr) + ")" + " - Gateway: " + defnic_gateway + " - Eth0 Connected: " + eth_connected_status + " - Eth0 Duplex: " + eth_duplex_status + " - Eth0 Speed: " + eth_speed_status)
			
	def refresh_external_ip_information(self):
		#Define External IPv4 addresses variable
		external_ipv4_function()

		#Define External IPv6 addresses variable
		external_ipv6_function()
			
		#Set new information
		self.labelipv4external.set_text(external_ipv4 + "\n" + external_ipv6)
			
		#Print info to console
		print ("Refreshed External IP's to:" + "\n" + "IPv4: " + external_ipv4 + "\n" + "IPv6: " + external_ipv6)
		#Add information to logfile
		logging.info("Refreshed External IP's to: IPv4: " + external_ipv4 + " - IPv6: " + external_ipv6)

	def refresh_ethernet_status_information(self):
		#Is ETH0 Connected
		eth_connected()
		
		#Define Duplex Mode
		eth_duplex()

		#Define Speed Mode
		eth_speed()

		#Set new information
		self.labelethernetinfo.set_text(eth_connected_status + "\n" + eth_duplex_status + "\n" + eth_speed_status)
		
		#Print info to console
		print ("Refreshed Ethernet connection status to:" + "\n" + "Ethernet Connected: " + eth_connected_status + "\n" + "Ethernet Duplex Status: " + eth_duplex_status + "\n" + "Ethernet Connection Speed: " + eth_speed_status)
		#Add information to logfile
		logging.info("Refreshed Ethernet connection status to:" + " - Ethernet Connected: " + eth_connected_status + " - Ethernet Duplex Status: " + eth_duplex_status + " - Ethernet Connection Speed: " + eth_speed_status)
		

	def refresh_ping_information(self):
		#Define if gateway is reachable / LAN is up
		gateway_reachable = str((ping_function(defnic_gateway, "1")))
			
		#Define if internet is reachable / WAN is up
		internet_reachable = str((ping_function("8.8.8.8", "1")))
			
		#Set new information
		self.labelconnectionup.set_text(gateway_reachable + "\n" + internet_reachable)
			
		#Print info to console
		print ("Refreshed Ping information:" + "\n" + "Gateway reachable? " + gateway_reachable + "\n" + "Internet Reachable? " + internet_reachable)
		#Add information to logfile
		logging.info("Refreshed Ping information:" + " - Gateway reachable? " + gateway_reachable + " - Internet Reachable? " + internet_reachable)

	def refresh_wifi_status_information(self):
		#Is WLAN0 Connected
		wifi_connected()
				
		#Define WiFI Channel
		wifi_channel()

		#Define WiFi Frequency
		wifi_frequency()

		#Define WiFi AccessPoint Mac
		wifi_accesspointmac()

		#Define WiFi BitRate
		wifi_bitrate()

		#Define WiFi Link Quality
		wifi_link_quality()
		
		#Set new information
		self.labelwifiinfo.set_text(wifi_connected_status + "\n" + wifi_channel_status + "\n" + wifi_frequency_status + "\n" + wifi_accesspointmac_status + "\n" + wifi_bitrate_status + "\n" + wifi_link_quality_status)
		
		#Print info to console
		print ("Refreshed WiFi connection status to:" + "\n" + "WLAN0 Connected: " + wifi_connected_status + "\n" + "WiFi Channel: " + wifi_channel_status + "\n" + "WiFi Frequency: " + wifi_frequency_status + "\n" + "WiFi AccessPoint MAC: " + wifi_accesspointmac_status + "\n" + "WiFi BitRate: " + wifi_bitrate_status + "\n" + "WiFi Link Quality: " + wifi_link_quality_status)
		#Add information to logfile
		logging.info("Refreshed WiFi connection status to:" + " - WLAN0 Connected: " + wifi_connected_status + " - WiFi Channel: " + wifi_channel_status + " - WiFi Frequency: " + wifi_frequency_status + " - WiFi AccessPoint MAC: " + wifi_accesspointmac_status + " - WiFi BitRate: " + wifi_bitrate_status + " - WiFi Link Quality: " + wifi_link_quality_status)
		
		
#End Jarno Class
		
		
		
###################
#Start the program#
###################

# If this is run stand alone execute the following after the 'if'
# If this class is imported into another program the code after the 'if' will
# not run. This makes the code more flexible.
if __name__ == "__main__":
	main = Jarno() # create an instance of our class
	Gtk.main() # run the program

